// package jtalk;

import java.io.*;
import java.net.*;
import java.util.*;

public class addressBook
{
	
    final static String addressBookFileName="jtalk.adb";
	final static String delimiter=",";
	final static int numberOfFieldsPerEntry=5;
	
    public static void saveData(Object[][] data, boolean DEBUG)
	{
        RandomAccessFile theRandomAccessFile=null;
        String temp;
	    
  		try  
        {   
            File theFile=new File(addressBookFileName);
            if(theFile.exists() && !theFile.delete())
            {
                if (DEBUG)
                    System.err.println("Address book couldn't be reset. Not updated. Aborted");
                return;
            }
            theRandomAccessFile= new RandomAccessFile(theFile, "rw"); 
		}
		catch(Exception exception)  
        {
          if (DEBUG)
            System.err.println("Error with opening address book file for writing. Due to :" +exception.toString());
        }
       	try
		{
    		theRandomAccessFile.writeBytes(constants.ADDRESS_BOOK_WARNING);
	    	theRandomAccessFile.writeBytes("\r\n");
		    for (int row=0; row<data.length; row++)
		    {
    			temp="";
			    int col=0;
	    		for (; col<numberOfFieldsPerEntry-1; col++)
    			{
		    		temp+=data[row][col].toString();
			    	temp+=delimiter;
			    }
			    temp+=data[row][col++].toString();
	      	    theRandomAccessFile.writeBytes(temp);
		        theRandomAccessFile.writeBytes("\r\n");
	        }
	    }
        catch(IOException exception)  
        {
            if (DEBUG)
                System.err.println("Couldn't write an entry to the address book file. Due to :"+exception.toString());   
        }   
		try
		{
    	    theRandomAccessFile.close();
        }
        catch(IOException exception)  
        {   
            if (DEBUG)
                System.err.println("Couldn't close address book file. Due to :"+exception.toString());   
        }   

    }
 
	public static Object[][] getData(boolean DEBUG)
	{
		Vector theVector=new Vector();
		RandomAccessFile theRandomAccessFile=null;
		Object data[][]=null;
		int col;
		
		try
		{
			theRandomAccessFile=new RandomAccessFile(addressBookFileName, "r");
		}
		catch (FileNotFoundException e)
		{
            if (DEBUG)
                System.err.println("Address book not found! Did you delete it? "+e);   
    		return null;
		}
		catch (IOException e)
		{
            if (DEBUG)
                System.err.println("IOException while geting Address Book data "+e);   
			return null;
		}
		try
		{
		    theRandomAccessFile.readLine();
			while(theRandomAccessFile.getFilePointer()<theRandomAccessFile.length())
			{
				theVector.addElement(theRandomAccessFile.readLine().trim()); 
			}
		}
		catch (IOException e)
		{
		}
		try
		{
			theRandomAccessFile.close();
		}
		catch (IOException e)
		{
		}
		data=new Object[theVector.size()][numberOfFieldsPerEntry];
		String token;
		for(int row=0; row<theVector.size(); row++)
		{
			col=0;
			StringTokenizer stringTokenizer=new StringTokenizer(theVector.elementAt(row).toString(), delimiter, true);
			while (stringTokenizer.hasMoreTokens())
			{
				token=stringTokenizer.nextToken();
				if (token.equals(delimiter))
					col++;
				else
				{
					if (col==numberOfFieldsPerEntry-1)
						data[row][col]=new Boolean(token);
					else
						data[row][col]=token;
				}
			}
		}
		return data;
	}

    public static void addEntry(Object[] data, boolean DEBUG)
    {
        if (data.length!=numberOfFieldsPerEntry)
        {
            if (DEBUG)
                System.err.println("Invalid entry! Not added.");
            return;
        }
        
        RandomAccessFile theRandomAccessFile=null;
  		try  
        {   
            theRandomAccessFile= new RandomAccessFile(addressBookFileName, "rw");   
            long offset=theRandomAccessFile.length();
	        theRandomAccessFile.seek(offset);  
		}
		catch(Exception exception)  
        {
            if (DEBUG)
                System.err.println("Error with opening address book file for writing. Due to :" +exception.toString());
        }
		
 	    String temp="";
		int col=0;
		for (; col<numberOfFieldsPerEntry-1; col++)
		{
			temp+=data[col].toString();
			temp+=delimiter;
		}
		temp+=data[col++].toString();
		try
		{
		    theRandomAccessFile.writeBytes(temp);
		    theRandomAccessFile.writeBytes("\r\n");
		}
        catch(IOException exception)  
        {   
            if (DEBUG)
                System.err.println("Couldn't add an entry to the address book file. Due to :"+exception.toString());   
        }   
		
       	try
		{
			theRandomAccessFile.close();
        }
        catch(IOException exception)  
        {   
            if (DEBUG)
                System.err.println("Couldn't close address book file. Due to :"+exception.toString());   
        }   
    }
    
    public static String[][] getNicks(boolean DEBUG)
    {
       	Object[][] data=getData(DEBUG);
        if (data==null ||data.length==0)
               return null;
        String[][]nicks;
       	Vector theVector=new Vector();
		String nick;
       	String address;
       	               
        for (int i=0; i<data.length; i++)
        {
            if (((Boolean)data[i][4]).booleanValue())
            {
                if (data[i][0]==null || data[i][0].equals(""))
                    continue;
                nick=data[i][0].toString();
                address=data[i][1].toString()+"@"+data[i][2].toString();
                theVector.addElement(nick);
                theVector.addElement(address);
            }
        }
        
        nicks=new String[theVector.size()/2][2];
		int x=0;
		for (int i=0; i<theVector.size()/2; i++)
		{
   		    nicks[i][0]=theVector.elementAt(x++).toString();
   		    nicks[i][1]=theVector.elementAt(x++).toString();
		}
		
       return nicks;
       
    }

    public static boolean compareHosts(String toHost, InetAddress[] dns, int mode)
    {
        if(dns!=null)
        {
            for (int j=0; j< dns.length; j++)
            {
                if (mode==1)
                {
                    if (dns[j].getHostName().startsWith(toHost))
                        return true;
                }
                else if (mode==-1)
                {
                    if (dns[j].getHostName().endsWith(toHost))
                        return true;
                }
                else 
                {
                    if (dns[j].getHostName().equals(toHost))
                        return true;
                }
            }
        }
        return false;
    }
    public static boolean hostEndsWithStar(String toHost, String host, InetAddress[] dns)
    {
        toHost=toHost.substring(0, toHost.length()-2);
        if (toHost.endsWith("."))
            toHost=toHost.substring(0, toHost.length()-2);
        if (toHost.equals(""))
            return true;
        return compareHosts(toHost, dns, 1);
    }
    
    public static boolean hostStartsWithStar(String toHost, String host, InetAddress[] dns)
    {
        toHost=toHost.substring(1);
        if (toHost.startsWith("."))
            toHost=toHost.substring(1);
        if (toHost.equals(""))
            return true;
        return compareHosts(toHost, dns, -1);
    }

    public static boolean belongsTo(String toUser, String toHost, String user, String host, InetAddress[] dns)
    {
        if (toUser.equals("*"))
        {
            if (toHost.equals("*"))
                return true;
            if (toHost.startsWith("*"))
            {
                return hostStartsWithStar(toHost, host, dns);
            }
            if (toHost.endsWith("*"))
            {
                return hostEndsWithStar(toHost, host, dns);
            }
            if (compareHosts(toHost, dns, 0))
                return true;
            return false;
         }
         else if (toUser.startsWith("*"))
         {
            toUser=toUser.substring(1);
            if (!user.endsWith(toUser))
                return false;
            else
            {
                if (toHost.equals("*"))
                    return true;
                if (toHost.startsWith("*"))
                {
                    return hostStartsWithStar(toHost, host, dns);
                }
                if (toHost.endsWith("*"))
                {
                    return hostEndsWithStar(toHost, host, dns);
                }
                if (compareHosts(toHost, dns, 0))
                    return true;
                return false;
            }
        }
        else if (toUser.endsWith("*"))
         {
            toUser=toUser.substring(0,toUser.length()-1);
            if (!user.startsWith(toUser))
                return false;
            else
            {
                if (toHost.equals("*"))
                    return true;
                if (toHost.startsWith("*"))
                {
                    return hostStartsWithStar(toHost, host, dns);
                }
                if (toHost.endsWith("*"))
                {
                    return hostEndsWithStar(toHost, host, dns);
                }
                if (compareHosts(toHost, dns, 0))
                    return true;
                return false;
            }
        }
        else
        {
            if (!user.equals(toUser))
            return false;
            else
            {
                if (toHost.equals("*"))
                    return true;
                if (toHost.startsWith("*"))
                {
                    return hostStartsWithStar(toHost, host, dns);
                }
                if (toHost.endsWith("*"))
                {
                    return hostEndsWithStar(toHost, host, dns);
                }
                if (compareHosts(toHost, dns, 0))
                    return true;
                return false;
            }
        }
    }
    
    public static int orderOf(String user, String host)
    {
        int order1=0;
        int order2=0;
        if (user.startsWith("*") || user.endsWith("*"))
        {
            order1=user.length();
        }
        else
        {
            order1=100*user.length();
        }
        StringTokenizer st=new StringTokenizer(host, ".", true);
        if (host.startsWith("*.") || host.endsWith(".*"))
        {
            order2=10000*(st.countTokens()-2);
        }
        else
        {
            order2=10000*(st.countTokens()-1);
        }
        
        return (order1+order2);
        
    }
    
    public static int getAction(String user, String host, boolean DEBUG)
    {
       	Object[][] data=getData(DEBUG);
       	int action=-1;
        if (data==null ||data.length==0)
               return action;
        InetAddress[] dns=null;
        try
        {
            dns=InetAddress.getAllByName(host);
        }
        catch(UnknownHostException e)
        {
        }
                       
               
        int curOrder=0;
        String curAction=null;
       	for (int i=0; i<data.length; i++)
        {
            if (((Boolean)data[i][4]).booleanValue())
            {
                String tempUser=data[i][1].toString();
                String tempHost=data[i][2].toString();
                if (belongsTo(tempUser, tempHost, user, host, dns))
                    if (orderOf(tempUser, tempHost)>curOrder)
                    {
                        curOrder=orderOf(tempUser, tempHost);
                        curAction=data[i][3].toString();
                    }
            }
        }
        if (curAction!=null)
        {
            for (int i=0; i<constants.ACTION_CHOICES.length; i++)
            {   
                if(curAction.equals(constants.ACTION_CHOICES[i]))
                {
                    action=i;
                    break;
                }
            }
        }
        return action;
    }
}