// package jtalk;

import java.awt.Component;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

public class jtableModel extends AbstractTableModel
{

    final private String[] columnNames;
	public Object[][] data;

	public jtableModel(String[] columnNames, Object[][] data)
	{
		this.columnNames=columnNames;
		this.data=data;
	}
     
    public int getColumnCount() 
	{
        return columnNames.length;
    }
	        
    public int getRowCount() 
	{
        return data.length;
    }
	
	public void addRow()
    {
	    addRow(null);
    }
        
    public void addRow(Object[] initialValues)
	{
	    int rows=getRowCount();
		int cols=getColumnCount();
        Object[][] temp=new Object[rows][cols];
		System.arraycopy(data, 0, temp, 0, rows);
		data=new Object[rows+1][cols];
		System.arraycopy(temp, 0, data, 0, rows);
		fireTableRowsInserted(rows, rows+1);
			
		fireTableChanged(new TableModelEvent(this, rows, rows+1, TableModelEvent.ALL_COLUMNS, TableModelEvent.INSERT)); 
		if (initialValues!=null)
		{
		    for (int col=0; col<Math.min(initialValues.length, getColumnCount()); col++)
		    {
    			data[rows][col]=initialValues[col];
			    fireTableCellUpdated(rows, col);
		    }
		}
    }			

	public void removeRow(int row) //from 0
	{
		int rows=getRowCount();
		int cols=getColumnCount();
		if (row>=rows)
		{
			return;
		}
		Object[][] temp=new Object[rows][cols];
		System.arraycopy(data, 0, temp, 0, rows);
		data=new Object[rows-1][cols];
		System.arraycopy(temp, 0, data, 0, row);
		System.arraycopy(temp, row+1, data, row, rows-row-1);
		fireTableRowsDeleted(row, row+1);
	}

	public String getColumnName(int col) 
	{
        return columnNames[col];
    }

    public Object getValueAt(int row, int col) 
	{
	    return data[row][col];
    }

    /**
     * JTable uses this method to determine the default renderer/
     * editor for each cell.  If we didn't implement this method,
     * then the last column would contain text ("true"/"false"),
     * rather than a check box.
     */
    
    public Class getColumnClass(int c) 
	{
        return getValueAt(0, c).getClass();
    }

    /**
     * Don't need to implement this method unless your table's
     * editable.
     */
         
    public boolean isCellEditable(int row, int col) 
	{
        //Note that the data/cell address is constant,
        //no matter where the cell appears onscreen.
        return false;
    }

    /*
     * Don't need to implement this method unless your table's
     * data can change.
     */
    
    public void setValueAt(Object value, int row, int col) 
	{
	    data[row][col] = value;
        fireTableCellUpdated(row, col);
    }
		
    /*
     * This method picks good column sizes.
     * If all column heads are wider than the column's cells' 
     * contents, then you can just use column.sizeWidthToFit().
     */
    
    public void initColumnSizes(JTable table) 
	{
        TableColumn column = null;
        Component comp = null;
        int headerWidth = 0;
        int cellWidth = 0;
        Object[] longValues=this.data[0];

        for (int i = 0; i < getColumnCount(); i++) 
		{
            column = table.getColumnModel().getColumn(i);
            comp = column.getHeaderRenderer().getTableCellRendererComponent(
                        null, column.getHeaderValue(), false, false, 0, 0);
            headerWidth = comp.getPreferredSize().width;
            comp = table.getDefaultRenderer(getColumnClass(i)).getTableCellRendererComponent(
                                 table, longValues[i], false, false, 0, i);
            cellWidth = comp.getPreferredSize().width;
            column.setMinWidth(Math.max(headerWidth, cellWidth));
        }
    }
	
	
	public void tableChanged(TableModelEvent e) 
	{
        fireTableChanged(e);
    }
    
    public void cellUpdated(int row, int col)
    {
        fireTableCellUpdated(row, col);
    }
    

}
    