// package jtalk;

import java.net.*;
import java.io.*;
import java.util.*;
import java.text.DateFormat;
import javax.swing.*;

public class jtalkd extends Thread
{
    public boolean DEBUG=false;
	private String localIP;
	private byte[] buffer=new byte[constants.CTL_MSG_LENGTH];
	private pendingRequestNode PendingRequests=null;
	private DatagramSocket theDatagramSocket;
	private volatile DatagramPacket inDatagramPacket;	
	private volatile DatagramPacket outDatagramPacket;	
	private volatile ctl_msg request;
	private volatile ctl_response response;
	private volatile int currentID = 0;	
	public static int talkMode = 0;	
    private JTalkDGUI theGUI;	
	
	public jtalkd(boolean DEBUG)
	{
	    this.DEBUG=DEBUG;
	    theGUI=new JTalkDGUI(this);
	}
	
	public void startDeamon()
	{
	    this.localIP=getIPorDNS.getLocalIP();
		try 
		{

			configurations theConfigurations=new configurations(DEBUG);
			theConfigurations.openConfigurationFile();
	
			if(theConfigurations.isValidConfigurationsFile())
			{
				theDatagramSocket = new DatagramSocket(((Integer)theConfigurations.getData("Advanced_Daemon_Port")).intValue());
			}
			else
			{
				theDatagramSocket = new DatagramSocket(constants.TALK_PORT);
			}
			theConfigurations.closeConfigurationFile();
			
			inDatagramPacket=new DatagramPacket(buffer, buffer.length);
			if (DEBUG)
			{
				System.err.println("JTALK DEAMON ("+localIP+") started on port: "+ theDatagramSocket.getLocalPort()+ " at: "+ DateFormat.getDateTimeInstance().format(new Date()));
			}
		}
		catch (SocketException se) 
		{
			if (DEBUG)
			{
				System.err.println("JTALK DEAMON ERROR! Can't open Datagram socket on port: "+ constants.TALK_PORT+ " at: "+ DateFormat.getDateTimeInstance().format(new Date()));
			}
			System.exit(1);
		}
		this.start();
	}
	
	public void stopDeamon()
	{
        if (DEBUG)
		{
			System.err.println("JTALK DEAMON stoped at: "+ DateFormat.getDateTimeInstance().format(new Date()));
		}
	    this.stop();
	    if (theDatagramSocket!=null)
	    {
	        PendingRequests=null;
	        inDatagramPacket=null;	
	        outDatagramPacket=null;	
	        theDatagramSocket.close();
	        theDatagramSocket=null;
	    }
	}
	
	public void run()
	{
		while (theDatagramSocket!=null) 
		{
			try 
			{	
				theDatagramSocket.receive(inDatagramPacket);
				if(inDatagramPacket.getLength()==constants.CTL_MSG_LENGTH)
				{
					request=new ctl_msg(inDatagramPacket.getData());
					if (DEBUG)
					{
						System.err.println("JTALK DEAMON. Recieved a request at: "+ DateFormat.getDateTimeInstance().format(new Date()));
						System.err.println(request.toString());
					}
					if ((talkMode==1 ||talkMode==2 || talkMode==5))
					{
					    if(talkMode==5)
					    {
					        response=null;
					    }
					    else
					    {
					        response=new ctl_response();
                            response.vers = constants.TALK_VERSION;
		                    response.type = request.type;
		                    response.id_num = request.id_num;
		                    response.addr.family = request.addr.family;
							    
					        if (talkMode==1)
                                response.answer=constants.NOT_HERE;
                            else if(talkMode==2)
                                response.answer=constants.PERMISSION_DENIED;
                        }
                    }
                    else
                    {
                        processRequest();
                    }
		        	if (response!=null)
				    {
        			    outDatagramPacket=new DatagramPacket( response.toBytes(), response.toBytes().length, inDatagramPacket.getAddress(), inDatagramPacket.getPort());
						theDatagramSocket.send(outDatagramPacket);
						if (DEBUG)
        			    {
						    System.err.println("JTALK DEAMON. Send respond at: "+ DateFormat.getDateTimeInstance().format(new Date()));
						    System.err.println(response.toString());
                	    }
					}
			    }
			}
 			catch (IOException e) 
			{
				if (DEBUG)
				{
					System.err.println("JTALK DEAMON ERROR! IOException in run at: "+ DateFormat.getDateTimeInstance().format(new Date()));
				}
				System.exit(1);
			}   
		}
	 } 

	public void processRequest()
	{
		response=new ctl_response();
		pendingRequestNode ptr;
		response.vers = constants.TALK_VERSION;
		response.type = request.type;
			
		if (request.vers != constants.TALK_VERSION) 
		{
			response.answer = constants.BADVERSION;
			return;
		}
		response.id_num = request.id_num;
		response.addr.family = request.addr.family;
		
		if (request.addr.family != constants.AF_INET) 
		{
			response.answer = constants.BADADDR;
			return;
		}
		switch (request.type) 
		{
			case constants.ANNOUNCE:
				announce();
			break;
		
			case constants.LEAVE_INVITE:
				ptr = findRequest();
				if (ptr != null) 
				{
					response.id_num = ptr.request.id_num;
					response.answer = constants.SUCCESS;
				}
				else
				{
		    		response.id_num = newID();
					insertEntry(false);
				}
			break;

			case constants.LOOK_UP:
				ptr = findMatch();
				if (ptr != null)
				{
					response.id_num = ptr.request.id_num;
					response.addr.family = ptr.request.addr.family;
					response.addr.port = ptr.request.addr.port;
					response.addr.ip = ptr.request.addr.ip;
					response.answer = constants.SUCCESS;
				}
				else
					response.answer = constants.NOT_HERE;
			break;
	
			case constants.DELETE:
				response.answer =(byte) deleteInvite(request.id_num);
			break;

			default:
				response.answer = constants.UNKNOWN_REQUEST;
			break;
		}
	}
	
		
/*
 * See if the user is 'logged on'. If so, announce that 
 * a talk is requested.
 */
	public void announce()
	{
		pendingRequestNode ptr;
		int result;
		/* see if the user is 'logged on'*/
		result = findUser();
		if (result != constants.SUCCESS) 
		{
			response.answer = (byte)result;
			return;
		}
		if (request.ctl_addr == null) 
		{
			response.answer = constants.MACHINE_UNKNOWN;
			return;
		}
		ptr = findRequest();
		if (ptr == null) 
		{
			/** announce that a talk is requested */
			insertEntry(true);
			print_mesg();
			response=null;
			return;
		}
		if (request.id_num > ptr.request.id_num) 
		{
		/**
		 * This is an explicit re-announce, so update the id_num
		 * field to avoid duplicates and re-announce the talk.
		 */
		 
		    ptr.request.id_num = newID();
			response.id_num =ptr.request.id_num;
			if(result==constants.SUCCESS)
			{
				if (DEBUG)
				    System.err.println("JTALK DEAMON. Re-Announce "+request);
				if(ptr.theTalkRequest.theRequest!=null)
				    ptr.theTalkRequest.theRequest.setVisible(true);
				response=null;
			}
			else
				response.answer =(byte) result;
			return;
		}
		else 
		{	
			/* a duplicated request, so ignore it */
			response.id_num = ptr.request.id_num;
			response=null;
			if (DEBUG)
			    System.err.println("JTALK DEAMON. Duplicated request. Ignored. "+request);
			response=null;
		}
	}
		
	
	/**
	* Insert a new entry into the top of the list and create new id
	*/
	public void insertEntry(boolean show)
	{
//		request.id_num=response.id_num;
		pendingRequestNode ptr = new pendingRequestNode(request, response, show, DEBUG);
		ptr.next = PendingRequests;
		PendingRequests = ptr;
	}

	
	/** Checks if the user is 'logged on'
	 */
	public int findUser()
	{
		String user=request.getCalleeName();
		boolean ACCEPT_ALL=true;
		boolean USE_SYSTEM_NAME=true;
		boolean USER_NOT_HERE=true;
		String USER_LIST="";
		configurations theConfigurations=new configurations(DEBUG);
		theConfigurations.openConfigurationFile();
		if(theConfigurations.isValidConfigurationsFile())
		{
			ACCEPT_ALL=((Boolean)theConfigurations.getData("Local_Accept_All")).booleanValue();
			USE_SYSTEM_NAME=((Boolean)theConfigurations.getData("Local_System_Name")).booleanValue();
			USER_NOT_HERE=((Boolean)theConfigurations.getData("Local_Unlisted_Users_Not_Here")).booleanValue();
			USER_LIST=(String)theConfigurations.getData("Local_Aliases");
		}
		theConfigurations.closeConfigurationFile();

		
		if (ACCEPT_ALL)
		{
			return constants.SUCCESS;
		}
		else
		{
		    boolean hasNicks=false;	
		    StringTokenizer stringTokenizer=new StringTokenizer(USER_LIST, " ");
		    while (stringTokenizer.hasMoreTokens())
		    {
		        hasNicks=true;
    			if (stringTokenizer.nextToken().equals(user))
					return constants.SUCCESS;
			}
			if(USE_SYSTEM_NAME || !hasNicks)
			{
				String systemUserName=System.getProperty("user.name");
				if (systemUserName.lastIndexOf(" ")>0)
					systemUserName=systemUserName.substring(systemUserName.lastIndexOf(" ")+1, systemUserName.length());
				if (systemUserName.equals(user))	
					return constants.SUCCESS;
			}
		}
		return USER_NOT_HERE?constants.NOT_HERE:constants.PERMISSION_DENIED;
	}


/*
 * Build a block of characters containing the message. 
 */
	public void print_mesg()
	{
		String message;
		String caller=new String(request.l_name);
		caller=caller.substring(0, caller.indexOf("\0"));
		String ip=getIPorDNS.dotlessToIP(request.addr.ip);
		
		
		message="\r\nMessage from JTalk_Daemon@"+localIP+" at ";

        message+=DateFormat.getDateTimeInstance().format(new Date())+"\n";
        message+="jtalk: connection requested by "+caller+"@"+ip+".\n";
		message+="jtalk: respond with: jtalk "+caller+"@"+ip+"\n";
		if (DEBUG)
		    System.err.println(message);
	}

	/**
	* Classic delete from a linked list
	*/
	public void delete(pendingRequestNode ptr)
	{
		if (PendingRequests == null)
		{
		    if (DEBUG)
			    System.err.println("There are no requests to delete.");
			return; 
		}
		if (PendingRequests == ptr)
		{
			PendingRequests = ptr.next;
			return;
		}
		for (pendingRequestNode temp = PendingRequests; temp.next != null; temp = temp.next) 
		{
			if (temp==ptr)
			{
				temp=temp.next;
				return;
			}	
		}
	}

/**
 * Look in the PendingRequests for an invitation that matches the current
 * request looking for an invitation
 */
	public pendingRequestNode findMatch()
	{
		if (PendingRequests==null)
			return null;
		pendingRequestNode ptr;
		long currentTime=System.currentTimeMillis();
		int MAX_LIFE;
		configurations theConfigurations=new configurations(DEBUG);
		theConfigurations.openConfigurationFile();
		if(theConfigurations.isValidConfigurationsFile())
		{
			MAX_LIFE=((Integer)theConfigurations.getData("Advanced_Max_Time")).intValue();
		}
		else
		{
			MAX_LIFE=constants.MAX_LIFE;
		}
		theConfigurations.closeConfigurationFile();
		
		for (ptr = PendingRequests; ptr != null; ptr = ptr.next) 
		{
			if ((ptr.time - currentTime) > (MAX_LIFE * 1000)) 
			{
				/* the entry is too old */
				delete(ptr);
				continue;
			}
			if (bytecmp(request.l_name, ptr.request.r_name) == 0 &&
				bytecmp(request.r_name, ptr.request.l_name) == 0 &&
				ptr.request.type == constants.LEAVE_INVITE)
					return (ptr);
		}
		return (null);
	}

/**
 * Look for an identical request, as opposed to a complimentary
 * one as findMatch does 
 */
	public pendingRequestNode findRequest()
	{
		pendingRequestNode ptr;
		long currentTime=System.currentTimeMillis();
	/*
	 * See if this is a repeated message, and check for
	 * out of date entries in the PendingRequests while we are it.
	 */
		int MAX_LIFE;
		configurations theConfigurations=new configurations(DEBUG);
		theConfigurations.openConfigurationFile();
		if(theConfigurations.isValidConfigurationsFile())
		{
			MAX_LIFE=((Integer)theConfigurations.getData("Advanced_Max_Time")).intValue();
		}
		else
		{
			MAX_LIFE=constants.MAX_LIFE;
		}
		theConfigurations.closeConfigurationFile();
		
		for (ptr = PendingRequests; ptr != null; ptr = ptr.next) 
		{
			if ((ptr.time - currentTime) > (MAX_LIFE * 1000)) 
			{
				/* the entry is too old */
				delete(ptr);
				continue;
			}
			if (bytecmp(request.r_name, ptr.request.r_name) == 0 &&
			  bytecmp(request.l_name, ptr.request.l_name) == 0 &&
				request.type == ptr.request.type &&
				request.pid == ptr.request.pid) 
				{
					/* update the time if we 'touch' it */
					ptr.time = currentTime;
					return (ptr);
				}
		}
		return (null);
	}


/**
 * Generate a unique non-zero sequence number
 */
	public int newID()
	{
		currentID = (currentID + 1) % constants.MAX_ID;
		/* 0 is reserved, helps to pick up bugs */
		if (currentID == 0)
			currentID = 1;
		return (currentID);
	}

/**
 * Delete the invitation with id 'id_num'
 */
	public int deleteInvite(int id_num)
	{
		pendingRequestNode ptr;
		ptr = PendingRequests;
		for (ptr = PendingRequests; ptr != null; ptr = ptr.next) 
		{
			if (ptr.request.id_num == id_num)
				break;
		}
		if (ptr != null) 
		{
			if((ptr.theTalkRequest.theRequest!=null) &&
				ptr.theTalkRequest.theRequest.isVisible())
			{
				ptr.theTalkRequest.theRequest.dispose();
			}
			delete(ptr);
			return (constants.SUCCESS);
		}
		return (constants.NOT_HERE);
	}
	
	public int bytecmp(byte[] first, byte[] second)
	{
		if (first.length != second.length)
			return 1;
		else
		{
			for (int i=0; i<first.length; i++)
			{
				if (first[i] != second[i])
					return 1;
			}
		}
		return 0;
	}

	public static void main(String[] args) 
	{ 
	    try 
		{
		    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} 
		catch (Exception e) 
		{ 
		}
		    boolean DEBUG;
		 	configurations theConfigurations=new configurations(false);
			theConfigurations.openConfigurationFile();
	
			if(theConfigurations.isValidConfigurationsFile())
			{
				DEBUG=((Boolean)theConfigurations.getData("Debug")).booleanValue();
				String debugFile=(String)theConfigurations.getData("Debug_File_Path");
				if (debugFile!=null && debugFile.length()>0)
				{
				    if (!debugFile.endsWith(File.separator))
			        {
    			        debugFile+=File.separator;
	    		    }
				    try
				    {
				       System.setErr(new PrintStream(new DataOutputStream(new FileOutputStream(debugFile+"jtalkd.dbg", true)), true));
				       String str="STARTING ERROR LOG AT: "+ DateFormat.getDateTimeInstance().format(new java.util.Date());
				       System.err.println();
				       System.err.print("\t+--");
				       for (int i=0; i<str.length(); i++)
           			        System.err.print("-");
           			   System.err.println("--+");
		               System.err.println("\t|  "+str+"  |");
				       System.err.print("\t+--");
				       for (int i=0; i<str.length(); i++)
           			        System.err.print("-");
           			   System.err.println("--+");
				    }
				    catch (IOException e)
				    {
				    }
			}
			}
			else
			{
				DEBUG=constants.DEBUG;
			}
			theConfigurations.closeConfigurationFile();
		  
		jtalkd theJtalk=new jtalkd(DEBUG);
		theJtalk.startDeamon();

	}
	
}

